import { __decorate, __extends, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import { isIE11, KeyCodes } from '@uifabric-vue/utilities';
import BaseComponent from '../BaseComponent';
var SELECTION_FORWARD = 'forward';
var SELECTION_BACKWARD = 'backward';
var Autofill = /** @class */ (function (_super) {
    __extends(Autofill, _super);
    function Autofill() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.autoFillEnabled = false;
        _this.isComposing = false;
        _this.internalValue = _this.defaultVisibleValue || '';
        _this.displayValue = _this.defaultVisibleValue || '';
        return _this;
    }
    Object.defineProperty(Autofill.prototype, "cursorLocation", {
        get: function () {
            if (this.$refs.inputElement) {
                var inputElement = this.$refs.inputElement;
                if (inputElement.selectionDirection !== SELECTION_FORWARD) {
                    return inputElement.selectionEnd;
                }
                else {
                    return inputElement.selectionStart;
                }
            }
            else {
                return -1;
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Autofill.prototype, "value", {
        get: function () {
            return this.internalValue;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Autofill.prototype, "isValueSelected", {
        get: function () {
            return Boolean(this.$refs.inputElement && this.$refs.inputElement.selectionStart !== this.$refs.inputElement.selectionEnd);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Autofill.prototype, "selectionStart", {
        get: function () {
            return this.$refs.inputElement ? this.$refs.inputElement.selectionStart : -1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Autofill.prototype, "selectionEnd", {
        get: function () {
            return this.$refs.inputElement ? this.$refs.inputElement.selectionEnd : -1;
        },
        enumerable: false,
        configurable: true
    });
    Autofill.prototype.focus = function () {
        this.$refs.inputElement && this.$refs.inputElement.focus();
    };
    Autofill.prototype.clear = function () {
        this.autoFillEnabled = true;
        this.updateValue('', false);
        this.$refs.inputElement && this.$refs.inputElement.setSelectionRange(0, 0);
    };
    Autofill.prototype.onKeyDown = function (ev) {
        this.$emit('keydown', ev);
        // If the event is actively being composed, then don't alert autofill.
        // Right now typing does not have isComposing, once that has been fixed any should be removed.
        if (!ev.isComposing) {
            switch (ev.which) {
                case KeyCodes.backspace:
                    this.autoFillEnabled = false;
                    break;
                case KeyCodes.left:
                case KeyCodes.right:
                    if (this.autoFillEnabled) {
                        this.internalValue = this.displayValue;
                        this.autoFillEnabled = false;
                    }
                    break;
                default:
                    if (!this.autoFillEnabled) {
                        if (this.enableAutofillOnKeyPress.indexOf(ev.which) !== -1) {
                            this.autoFillEnabled = true;
                        }
                    }
                    break;
            }
        }
    };
    ;
    Autofill.prototype.onInputChanged = function (ev) {
        var value = this.getCurrentInputValue(ev);
        if (!this.isComposing) {
            this.tryEnableAutofill(value, this.value, ev.isComposing);
        }
        // If it is not IE11 and currently composing, update the value
        if (!(isIE11() && this.isComposing)) {
            var nativeEventComposing = ev.composed;
            var isComposing = nativeEventComposing === undefined ? this.isComposing : nativeEventComposing;
            this.updateValue(value, isComposing);
        }
    };
    ;
    Autofill.prototype.getCurrentInputValue = function (ev) {
        if (ev && ev.target && ev.target.value) {
            return ev.target.value;
        }
        else if (this.$refs.inputElement && this.$refs.inputElement.value) {
            return this.$refs.inputElement.value;
        }
        else {
            return '';
        }
    };
    /**
     * Updates the current input value as well as getting a new display value.
     * @param newValue - The new value from the input
     */
    Autofill.prototype.updateValue = function (newValue, composing) {
        // Only proceed if the value is nonempty and is different from the old value
        // This is to work around the fact that, in IE 11, inputs with a placeholder fire an onInput event on focus
        if (!newValue && newValue === this.value) {
            return;
        }
        this.internalValue = this.onInputChange ? this.onInputChange(newValue, composing) : newValue;
        this.displayValue = this.getDisplayValue(this.value, this.suggestedDisplayValue);
    };
    Autofill.prototype.getDisplayValue = function (inputValue, suggestedDisplayValue) {
        var displayValue = inputValue;
        if (suggestedDisplayValue &&
            inputValue &&
            this.doesTextStartWith(suggestedDisplayValue, displayValue) &&
            this.autoFillEnabled) {
            displayValue = suggestedDisplayValue;
        }
        return displayValue;
    };
    Autofill.prototype.doesTextStartWith = function (text, startWith) {
        if (!text || !startWith) {
            return false;
        }
        return text.toLocaleLowerCase().indexOf(startWith.toLocaleLowerCase()) === 0;
    };
    /**
     * Attempts to enable autofill. Whether or not autofill is enabled depends on the input value,
     * whether or not any text is selected, and only if the new input value is longer than the old input value.
     * Autofill should never be set to true if the value is composing. Once compositionEnd is called, then
     * it should be completed.
     * See https://developer.mozilla.org/en-US/docs/Web/API/CompositionEvent for more information on composition.
     * @param newValue - new input value
     * @param oldValue - old input value
     * @param isComposing - if true then the text is actively being composed and it has not completed.
     * @param isComposed - if the text is a composed text value.
     */
    Autofill.prototype.tryEnableAutofill = function (newValue, oldValue, isComposing, isComposed) {
        if (!isComposing &&
            newValue &&
            this.$refs.inputElement &&
            this.$refs.inputElement.selectionStart === newValue.length &&
            !this.autoFillEnabled &&
            (newValue.length > oldValue.length || isComposed)) {
            this.autoFillEnabled = true;
        }
    };
    Autofill.prototype.render = function (h) {
        var _this = this;
        var displayValue = this.displayValue;
        return h('input', {
            ref: 'inputElement',
            attrs: {
                autoCapitalize: 'off',
                autoComplete: 'off',
                'aria-autocomplete': 'both',
                value: displayValue,
            },
            on: {
                change: this.onInputChanged,
                click: function (ev) { return _this.$emit('click', ev); },
            },
        });
    };
    __decorate([
        Prop({ type: String, default: '' }),
        __metadata("design:type", String)
    ], Autofill.prototype, "defaultVisibleValue", void 0);
    __decorate([
        Prop({ type: String, default: '' }),
        __metadata("design:type", String)
    ], Autofill.prototype, "suggestedDisplayValue", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], Autofill.prototype, "preventValueSelection", void 0);
    __decorate([
        Prop({ type: Array, default: function () { return [KeyCodes.down, KeyCodes.up]; } }),
        __metadata("design:type", Array)
    ], Autofill.prototype, "enableAutofillOnKeyPress", void 0);
    __decorate([
        Prop({ type: Function, default: null }),
        __metadata("design:type", Function)
    ], Autofill.prototype, "onInputChange", void 0);
    __decorate([
        Prop({ type: Function, default: null }),
        __metadata("design:type", Function)
    ], Autofill.prototype, "onInputValueChange", void 0);
    Autofill = __decorate([
        Component
    ], Autofill);
    return Autofill;
}(BaseComponent));
export { Autofill };
//# sourceMappingURL=Autofill.js.map