import { __decorate, __extends, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import BaseComponent from '../BaseComponent';
import { getBaseButtonClassNames } from './BaseButton.classNames';
import { Icon, FontIcon } from '../Icon';
import { DirectionalHint } from '../../common/DirectionalHint';
import { ContextualMenu } from '../ContextualMenu';
var TouchIdleDelay = 500; /* ms */
var BaseButton = /** @class */ (function (_super) {
    __extends(BaseButton, _super);
    function BaseButton() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.menuHidden = true;
        _this.menuOpen = false;
        return _this;
    }
    Object.defineProperty(BaseButton.prototype, "MenuType", {
        get: function () {
            return this.menuAs || ContextualMenu;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseButton.prototype, "component", {
        get: function () {
            var _a = this, disabled = _a.disabled, href = _a.href;
            var renderAsAnchor = !disabled && !!href;
            return renderAsAnchor ? 'a' : 'button';
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseButton.prototype, "isSplitButton", {
        get: function () {
            return !!this.menuProps && this.split === true;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseButton.prototype, "shouldRenderMenu", {
        get: function () {
            var menuHidden = this.menuHidden;
            var _a = this, persistMenu = _a.persistMenu, renderPersistedMenuHiddenOnMount = _a.renderPersistedMenuHiddenOnMount;
            if (!menuHidden) {
                // Always should render a menu when it is expanded
                return true;
            }
            else if (persistMenu && (false || renderPersistedMenuHiddenOnMount)) {
                // _renderedVisibleMenu ensures that the first rendering of
                // the menu happens on-screen, as edge's scrollbar calculations are off if done while hidden.
                return true;
            }
            return false;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseButton.prototype, "classNames", {
        get: function () {
            var _a = this, theme = _a.theme, styles = _a.styles, className = _a.className, iconProps = _a.iconProps, menuIconProps = _a.menuIconProps, variantClassName = _a.variantClassName, disabled = _a.disabled, primaryDisabled = _a.primaryDisabled, checked = _a.checked, getClassNames = _a.getClassNames;
            var isPrimaryButtonDisabled = disabled || primaryDisabled;
            return getClassNames
                ? getClassNames(theme, className, variantClassName, iconProps && iconProps.className, menuIconProps && menuIconProps.className, isPrimaryButtonDisabled, checked, !this.menuHidden, !!this.menuProps, this.split, !!this.allowDisabledFocus)
                : getBaseButtonClassNames(theme, styles, className, variantClassName, iconProps && iconProps.className, menuIconProps && menuIconProps.className, isPrimaryButtonDisabled, !!this.menuProps, checked, !this.menuHidden, this.split);
        },
        enumerable: false,
        configurable: true
    });
    BaseButton.prototype.render = function (h) {
        var _this = this;
        var ButtonComponent = this.component;
        var MenuComponent = this.MenuType;
        var _a = this, classNames = _a.classNames, href = _a.href, iconProps = _a.iconProps, className = _a.className, css = _a.css, secondaryText = _a.secondaryText, isSplitButton = _a.isSplitButton, menuProps = _a.menuProps, menuIconProps = _a.menuIconProps, shouldRenderMenu = _a.shouldRenderMenu;
        return h(ButtonComponent, {
            ref: 'buttonElement',
            class: classNames.root,
            attrs: { href: href },
            on: {
                click: function (ev) { return _this.$emit('click', ev); },
            },
        }, [
            h('span', { class: classNames.flexContainer }, [
                iconProps && h(Icon, {
                    class: css(classNames.icon, className),
                    props: iconProps,
                }),
                this.$scopedSlots.flex && this.$scopedSlots.flex({}),
                this.$scopedSlots.default && h('span', { class: classNames.textContainer }, [
                    h('span', { class: classNames.label }, this.$scopedSlots.default({})),
                    secondaryText && h('span', { class: classNames.description }, secondaryText),
                ]),
                (!isSplitButton && menuProps) && h(FontIcon, {
                    class: classNames.menuIcon,
                    attrs: {
                        iconName: 'ChevronDown',
                    },
                    props: menuIconProps,
                }),
                (menuProps && !menuProps.doNotLayer && shouldRenderMenu) && h(MenuComponent, {
                    class: css('ms-BaseButton-menuhost', menuProps.className),
                    attrs: {
                        directionalHint: DirectionalHint.bottomLeftEdge,
                        target: isSplitButton ? this.$refs.splitButtonContainer : this.$refs.buttonElement,
                    },
                    props: menuProps,
                    on: {
                        dismiss: function () { return (_this.menuHidden = true); },
                    },
                }),
            ]),
        ]);
    };
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], BaseButton.prototype, "href", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], BaseButton.prototype, "checked", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], BaseButton.prototype, "split", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], BaseButton.prototype, "disabled", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], BaseButton.prototype, "primaryDisabled", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], BaseButton.prototype, "allowDisabledFocus", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], BaseButton.prototype, "variantClassName", void 0);
    __decorate([
        Prop({ type: Object, default: function () { } }),
        __metadata("design:type", Object)
    ], BaseButton.prototype, "iconProps", void 0);
    __decorate([
        Prop({ type: Object, default: function () { } }),
        __metadata("design:type", Object)
    ], BaseButton.prototype, "menuIconProps", void 0);
    __decorate([
        Prop({ type: Object, default: null }),
        __metadata("design:type", Object)
    ], BaseButton.prototype, "menuAs", void 0);
    __decorate([
        Prop({ type: Object, default: null }),
        __metadata("design:type", Object)
    ], BaseButton.prototype, "menuProps", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], BaseButton.prototype, "persistMenu", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], BaseButton.prototype, "renderPersistedMenuHiddenOnMount", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], BaseButton.prototype, "secondaryText", void 0);
    __decorate([
        Prop({ type: Function, default: null }),
        __metadata("design:type", Function)
    ], BaseButton.prototype, "getClassNames", void 0);
    BaseButton = __decorate([
        Component
    ], BaseButton);
    return BaseButton;
}(BaseComponent));
export { BaseButton };
//# sourceMappingURL=BaseButton.js.map