import { __assign, __decorate, __extends, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import BaseComponent from '../BaseComponent';
import { classNamesFunction } from '@uifabric-vue/utilities';
import { positionCallout } from '../../utilities/positioning';
import { clickedOutside } from '../../utilities/clickedOutside';
var getClassNames = classNamesFunction();
var CalloutContentBase = /** @class */ (function (_super) {
    __extends(CalloutContentBase, _super);
    function CalloutContentBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.internalKey = new Date();
        _this.positions = null;
        _this.positionAttempts = 0;
        return _this;
    }
    CalloutContentBase.prototype.created = function () {
        window.addEventListener('click', this.onGlobalClick, true);
        window.addEventListener('scroll', this.onGlobalScroll, true);
    };
    CalloutContentBase.prototype.beforeDestroy = function () {
        window.removeEventListener('click', this.onGlobalClick, true);
        window.removeEventListener('scroll', this.onGlobalScroll, true);
    };
    CalloutContentBase.prototype.updated = function () {
        this.internalKey = new Date();
    };
    CalloutContentBase.prototype.mounted = function () {
        this.updatePosition();
    };
    Object.defineProperty(CalloutContentBase.prototype, "classNames", {
        get: function () {
            var _a = this, theme = _a.theme, className = _a.className, calloutMaxWidth = _a.calloutMaxWidth, backgroundColor = _a.backgroundColor, finalHeight = _a.finalHeight, positions = _a.positions, styles = _a.styles, calloutWidth = _a.calloutWidth, beakWidth = _a.beakWidth;
            return getClassNames(styles, {
                theme: theme,
                className: className,
                overflowYHidden: !!finalHeight,
                calloutWidth: calloutWidth,
                positions: positions,
                beakWidth: beakWidth,
                backgroundColor: backgroundColor,
                calloutMaxWidth: calloutMaxWidth,
            });
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CalloutContentBase.prototype, "actualBeakWidth", {
        get: function () {
            return Math.sqrt(this.beakWidth * this.beakWidth * 2);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CalloutContentBase.prototype, "positionCss", {
        get: function () {
            if (!this.positions)
                return null;
            var positionCss = this.positions;
            for (var key in positionCss.elementPosition) {
                positionCss.elementPosition[key] = positionCss.elementPosition[key] + "px";
            }
            for (var key in positionCss.beakPosition.elementPosition) {
                positionCss.beakPosition.elementPosition[key] = positionCss.beakPosition.elementPosition[key] + "px";
            }
            return positionCss;
        },
        enumerable: false,
        configurable: true
    });
    CalloutContentBase.prototype.updatePosition = function () {
        var positions = this.positions;
        var currentProps = __assign({}, this.$props);
        currentProps.bounds = this.bounds;
        currentProps.target = this.target;
        var newPositions = positionCallout(currentProps, this.$refs.hostElement, this.$refs.calloutElement, this.positions);
        if ((!positions && newPositions) ||
            (positions && newPositions && !this._arePositionsEqual(positions, newPositions) && this.positionAttempts < 5)) {
            this.positionAttempts++;
            this.positions = newPositions;
        }
        else if (this.positionAttempts > 0) {
            this.positionAttempts = 0;
            this.$emit('positioned', this.positions);
        }
    };
    CalloutContentBase.prototype.onGlobalClick = function (e) {
        var outside = clickedOutside(e, this.$refs.calloutElement);
        if (outside)
            this.$emit('dismiss', true);
    };
    CalloutContentBase.prototype.onGlobalScroll = function (e) {
        this.$emit('dismiss', true);
    };
    Object.defineProperty(CalloutContentBase.prototype, "bounds", {
        get: function () {
            return {
                top: 0 + this.minPagePadding,
                left: 0 + this.minPagePadding,
                right: window.innerWidth - this.minPagePadding,
                bottom: window.innerHeight - this.minPagePadding,
                width: window.innerWidth - this.minPagePadding * 2,
                height: window.innerHeight - this.minPagePadding * 2,
            };
        },
        enumerable: false,
        configurable: true
    });
    CalloutContentBase.prototype._arePositionsEqual = function (positions, newPosition) {
        return (this._comparePositions(positions.elementPosition, newPosition.elementPosition) &&
            this._comparePositions(positions.beakPosition.elementPosition, newPosition.beakPosition.elementPosition));
    };
    CalloutContentBase.prototype._comparePositions = function (oldPositions, newPositions) {
        for (var key in newPositions) {
            // This needs to be checked here and below because there is a linting error if for in does not immediately have an if statement
            if (newPositions.hasOwnProperty(key)) {
                var oldPositionEdge = oldPositions[key];
                var newPositionEdge = newPositions[key];
                if (oldPositionEdge !== undefined && newPositionEdge !== undefined) {
                    if (oldPositionEdge.toFixed(2) !== newPositionEdge.toFixed(2)) {
                        return false;
                    }
                }
                else {
                    return false;
                }
            }
        }
        return true;
    };
    CalloutContentBase.prototype.render = function (h) {
        var _a = this, classNames = _a.classNames, positionCss = _a.positionCss, isBeakVisible = _a.isBeakVisible;
        return h('div', {
            ref: 'hostElement',
            class: classNames.container,
        }, [
            h('div', {
                ref: 'calloutElement',
                class: classNames.root,
                style: positionCss ? positionCss.elementPosition : null,
            }, [
                isBeakVisible && h('div', {
                    class: classNames.beak,
                    style: (positionCss && positionCss.beakPosition) ? positionCss.beakPosition.elementPosition : null,
                }),
                isBeakVisible && h('div', { class: classNames.beakCurtain }),
                h('div', { class: classNames.calloutMain }, [
                    this.$slots.default,
                ]),
            ]),
        ]);
    };
    __decorate([
        Prop({ type: HTMLElement, required: true }),
        __metadata("design:type", HTMLElement)
    ], CalloutContentBase.prototype, "target", void 0);
    __decorate([
        Prop({ type: Number, default: null }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "calloutWidth", void 0);
    __decorate([
        Prop({ type: Number, default: 16 }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "beakWidth", void 0);
    __decorate([
        Prop({ type: Number, default: 8 }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "minPagePadding", void 0);
    __decorate([
        Prop({ type: Boolean, default: true }),
        __metadata("design:type", Boolean)
    ], CalloutContentBase.prototype, "isBeakVisible", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CalloutContentBase.prototype, "coverTarget", void 0);
    __decorate([
        Prop({ type: Number, default: 0 }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "gapSpace", void 0);
    __decorate([
        Prop({ type: Number, default: undefined }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "finalHeight", void 0);
    __decorate([
        Prop({ type: Number, default: undefined }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "calloutMaxWidth", void 0);
    __decorate([
        Prop({ type: String, default: undefined }),
        __metadata("design:type", String)
    ], CalloutContentBase.prototype, "backgroundColor", void 0);
    CalloutContentBase = __decorate([
        Component
    ], CalloutContentBase);
    return CalloutContentBase;
}(BaseComponent));
export { CalloutContentBase };
//# sourceMappingURL=CalloutContent.base.js.map