import { __assign, __decorate, __extends, __metadata } from "tslib";
import { Component, Prop, Watch, Model } from 'vue-property-decorator';
import { Label } from '../../Label';
import { Icon } from '../../Icon';
import BaseComponent from '../BaseComponent';
import { classNamesFunction } from '@uifabric-vue/utilities';
var getClassNames = classNamesFunction();
var CheckboxBase = /** @class */ (function (_super) {
    __extends(CheckboxBase, _super);
    function CheckboxBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.internalValue = _this.checked || _this.defaultChecked;
        _this.isIndeterminate = _this.indeterminate || _this.defaultIndeterminate;
        return _this;
    }
    CheckboxBase.prototype.onCheckedChanged = function (newVal) {
        this.internalValue = newVal;
    };
    CheckboxBase.prototype.render = function (h) {
        var _a = this, classNames = _a.classNames, title = _a.title, label = _a.label;
        var $input = h('input', {
            class: classNames.input,
            attrs: __assign(__assign({ id: "Checkbox" + this.uid }, this.$attrs), { disabled: this.disabled, type: 'checkbox' }),
            on: {
                input: this.onInput,
            },
        });
        var $icon = h(Icon, {
            class: classNames.checkmark,
            attrs: {
                iconName: 'CheckMark',
            },
        });
        var $checkbox = h('div', { class: classNames.checkbox }, [$icon]);
        var $text = (label || this.$slots.default) && h('span', {
            class: classNames.text,
            attrs: {
                title: title,
            },
        }, this.$slots.default || label);
        return h('div', { class: classNames.root }, [
            $input,
            h(Label, {
                class: classNames.label,
                attrs: { for: "Checkbox" + this.uid },
            }, [
                $checkbox,
                $text,
            ]),
        ]);
    };
    Object.defineProperty(CheckboxBase.prototype, "classNames", {
        get: function () {
            var _a = this, theme = _a.theme, className = _a.className, disabled = _a.disabled, isIndeterminate = _a.isIndeterminate, internalValue = _a.internalValue, boxSide = _a.boxSide;
            return getClassNames(this.styles, {
                theme: theme,
                className: className,
                disabled: disabled,
                indeterminate: isIndeterminate,
                checked: internalValue,
                reversed: boxSide !== 'start',
                isUsingCustomLabelRender: true,
            });
        },
        enumerable: false,
        configurable: true
    });
    CheckboxBase.prototype.onInput = function () {
        if (this.disabled)
            return;
        if (this.isIndeterminate) {
            this.internalValue = this.defaultChecked;
            this.isIndeterminate = false;
        }
        else {
            this.internalValue = !this.internalValue;
        }
        this.$emit('input', this.internalValue);
    };
    __decorate([
        Model('input', { type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CheckboxBase.prototype, "checked", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CheckboxBase.prototype, "defaultChecked", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CheckboxBase.prototype, "indeterminate", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CheckboxBase.prototype, "defaultIndeterminate", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CheckboxBase.prototype, "disabled", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CheckboxBase.prototype, "required", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], CheckboxBase.prototype, "label", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], CheckboxBase.prototype, "title", void 0);
    __decorate([
        Prop({ type: String, default: 'start', validator: function (v) { return ['start', 'end'].indexOf(v) > -1; } }),
        __metadata("design:type", String)
    ], CheckboxBase.prototype, "boxSide", void 0);
    __decorate([
        Watch('checked'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Boolean]),
        __metadata("design:returntype", void 0)
    ], CheckboxBase.prototype, "onCheckedChanged", null);
    CheckboxBase = __decorate([
        Component({})
    ], CheckboxBase);
    return CheckboxBase;
}(BaseComponent));
export { CheckboxBase };
//# sourceMappingURL=Checkbox.base.js.map