import { __decorate, __extends, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import { MountingPortal } from 'portal-vue';
import { registerLayer, getDefaultTarget, unregisterLayer } from './Layer.notification';
import BaseComponent from '../BaseComponent';
import { getStyles } from './Layer.styles';
import { classNamesFunction } from '@uifabric-vue/utilities';
var getClassNames = classNamesFunction();
var LayerBase = /** @class */ (function (_super) {
    __extends(LayerBase, _super);
    function LayerBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.hasTarget = false;
        return _this;
    }
    Object.defineProperty(LayerBase.prototype, "classNames", {
        get: function () {
            var _a = this, className = _a.className, theme = _a.theme;
            return getClassNames(getStyles, {
                theme: theme,
                className: className,
                isNotHost: !this.hostId,
            });
        },
        enumerable: false,
        configurable: true
    });
    LayerBase.prototype.mounted = function () {
        this.createElement();
        if (this.hostId) {
            registerLayer(this.hostId, this.createElement);
        }
    };
    LayerBase.prototype.beforeDestroy = function () {
        var hostId = this.hostId;
        if (this.hostId) {
            unregisterLayer(this.hostId, this.createElement);
        }
    };
    LayerBase.prototype.createElement = function () {
        var doc = this.$el.ownerDocument;
        var host = this.getHost();
        if (!doc || !host) {
            this.hasTarget = false;
            return;
        }
        this.hasTarget = true;
    };
    LayerBase.prototype.getHost = function () {
        var doc = this.$el.ownerDocument;
        if (!doc) {
            return undefined;
        }
        if (this.hostId) {
            return doc.getElementById(this.hostId);
        }
        else {
            var defaultHostSelector = getDefaultTarget();
            return defaultHostSelector
                ? doc.querySelector(defaultHostSelector)
                : doc.body;
        }
    };
    LayerBase.prototype.render = function (h) {
        if (!this.hasTarget)
            return;
        var _a = this, classNames = _a.classNames, hostId = _a.hostId, append = _a.append;
        return h(MountingPortal, {
            props: {
                mountTo: hostId ? "#" + hostId : 'body',
                append: append,
            },
        }, [
            h('div', { class: classNames.root }, [
                h('div', { class: classNames.content }, [
                    this.$scopedSlots.default && this.$scopedSlots.default({}),
                ]),
            ]),
        ]);
    };
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], LayerBase.prototype, "hostId", void 0);
    __decorate([
        Prop({ type: Boolean, default: true }),
        __metadata("design:type", Boolean)
    ], LayerBase.prototype, "append", void 0);
    LayerBase = __decorate([
        Component
    ], LayerBase);
    return LayerBase;
}(BaseComponent));
export { LayerBase };
//# sourceMappingURL=Layer.base.js.map