import { __decorate, __extends, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import { getDocument, getWindow, on, doesElementContainFocus, KeyCodes } from '@uifabric-vue/utilities';
import BaseComponent from '../BaseComponent';
var Popup = /** @class */ (function (_super) {
    __extends(Popup, _super);
    function Popup() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.needsVerticalScrollBar = false;
        _this._containsFocus = false;
        return _this;
    }
    Popup.prototype.created = function () {
        this._originalFocusedElement = getDocument().activeElement;
    };
    Popup.prototype.mounted = function () {
        if (this.$refs.current) {
            this._disposables.push(
            // @ts-ignore
            on(this.$refs.current, 'focus', this._onFocus, true), 
            // @ts-ignore
            on(this.$refs.current, 'blur', this._onBlur, true));
            var currentWindow = getWindow(this.$refs.current);
            if (currentWindow) {
                // @ts-ignore
                this._disposables.push(on(currentWindow, 'keydown', this._onKeyDown));
            }
            if (doesElementContainFocus(this.$refs.current)) {
                this._containsFocus = true;
            }
        }
        this._updateScrollBarAsync();
    };
    Popup.prototype.updated = function () {
        this._updateScrollBarAsync();
        this._async.dispose();
    };
    Popup.prototype.beforeDestroy = function () {
        if (this.shouldRestoreFocus &&
            this._originalFocusedElement &&
            this._containsFocus &&
            this._originalFocusedElement !== window) {
            // This slight delay is required so that we can unwind the stack, let react try to mess with focus, and then
            // apply the correct focus. Without the setTimeout, we end up focusing the correct thing, and then React wants
            // to reset the focus back to the thing it thinks should have been focused.
            if (this._originalFocusedElement) {
                this._originalFocusedElement.focus();
            }
        }
    };
    Popup.prototype.render = function (h, context) {
        var className = this.className;
        return h('div', {
            ref: 'current',
            class: className,
            style: { overflowY: this.needsVerticalScrollBar ? 'scroll' : undefined, outline: 'none' },
            on: {
                keydown: this._onKeyDown,
            },
        }, this.$slots.default);
    };
    Popup.prototype._onKeyDown = function (ev) {
        switch (ev.which) {
            case KeyCodes.escape:
                if (this.onDismiss) {
                    this.onDismiss(ev);
                    ev.preventDefault();
                    ev.stopPropagation();
                }
                break;
        }
    };
    ;
    Popup.prototype._updateScrollBarAsync = function () {
        var _this = this;
        this._async.requestAnimationFrame(function () {
            _this._getScrollBar();
        });
    };
    Popup.prototype._getScrollBar = function () {
        // If overflowY is overriden, don't waste time calculating whether the scrollbar is necessary.
        // TODO remove ts-ignore and fix `this.style`
        // @ts-ignore
        if (this.style && this.style.overflowY) {
            return;
        }
        var needsVerticalScrollBar = false;
        if (this.$refs.current && this.$refs.current.firstElementChild) {
            // ClientHeight returns the client height of an element rounded to an
            // integer. On some browsers at different zoom levels this rounding
            // can generate different results for the root container and child even
            // though they are the same height. This causes us to show a scroll bar
            // when not needed. Ideally we would use BoundingClientRect().height
            // instead however seems that the API is 90% slower than using ClientHeight.
            // Therefore instead we will calculate the difference between heights and
            // allow for a 1px difference to still be considered ok and not show the
            // scroll bar.
            var rootHeight = this.$refs.current.clientHeight;
            var firstChildHeight = this.$refs.current.firstElementChild.clientHeight;
            if (rootHeight > 0 && firstChildHeight > rootHeight) {
                needsVerticalScrollBar = firstChildHeight - rootHeight > 1;
            }
        }
        if (this.needsVerticalScrollBar !== needsVerticalScrollBar) {
            this.needsVerticalScrollBar = needsVerticalScrollBar;
        }
    };
    Popup.prototype._onFocus = function () {
        this._containsFocus = true;
    };
    ;
    Popup.prototype._onBlur = function (ev) {
        if (this.$refs.current && this.$refs.current.contains(ev.relatedTarget)) {
            this._containsFocus = false;
        }
    };
    ;
    __decorate([
        Prop({ type: Boolean, default: true }),
        __metadata("design:type", Boolean)
    ], Popup.prototype, "shouldRestoreFocus", void 0);
    __decorate([
        Prop({ type: Function, default: null }),
        __metadata("design:type", Function)
    ], Popup.prototype, "onDismiss", void 0);
    Popup = __decorate([
        Component({
            components: {},
        })
    ], Popup);
    return Popup;
}(BaseComponent));
export { Popup };
//# sourceMappingURL=Popup.js.map