import { __assign, __decorate, __extends, __metadata } from "tslib";
import { Component, Prop, Watch } from 'vue-property-decorator';
import { Icon } from '../Icon';
import { IconButton } from '../Button';
import BaseComponent from '../BaseComponent';
import { classNamesFunction, KeyCodes } from '@uifabric-vue/utilities';
var getClassNames = classNamesFunction();
var SearchBoxBase = /** @class */ (function (_super) {
    __extends(SearchBoxBase, _super);
    function SearchBoxBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isActive = false;
        _this.internalValue = _this.value;
        return _this;
    }
    SearchBoxBase.prototype.render = function (h) {
        var _a = this, classNames = _a.classNames, disabled = _a.disabled, internalValue = _a.internalValue, placeholder = _a.placeholder, iconName = _a.iconName;
        var $iconContainer = h('div', { class: classNames.iconContainer }, [
            h(Icon, { class: classNames.icon, props: { iconName: iconName } }),
        ]);
        var $clearButton = internalValue && h('div', { class: classNames.clearButton }, [
            h(IconButton, {
                props: {
                    styles: { root: { height: 'auto' }, icon: { fontSize: '12px' } },
                    iconProps: { iconName: 'Clear' },
                },
                on: { click: this.clearInput },
            }),
        ]);
        var $input = h('input', {
            ref: 'input',
            class: classNames.field,
            attrs: __assign(__assign({}, this.$attrs), { disabled: disabled, value: internalValue, 'aria-label': placeholder, placeholder: placeholder }),
            on: {
                input: this.onInput,
                focus: this.onFocus,
                blur: this.onBlur,
                keydown: this.onKeyDown,
            },
        });
        return h('div', { class: classNames.root }, [
            $iconContainer,
            $input,
            $clearButton,
        ]);
    };
    Object.defineProperty(SearchBoxBase.prototype, "classNames", {
        get: function () {
            var _a = this, theme = _a.theme, underlined = _a.underlined, disabled = _a.disabled, isActive = _a.isActive, className = _a.className, disableAnimation = _a.disableAnimation, internalValue = _a.internalValue;
            return getClassNames(this.styles, {
                theme: theme,
                className: className,
                underlined: underlined,
                hasFocus: isActive,
                disabled: disabled,
                hasInput: internalValue == null ? false : internalValue.length > 0,
                disableAnimation: disableAnimation,
            });
        },
        enumerable: false,
        configurable: true
    });
    SearchBoxBase.prototype.onPropValueChanged = function (newValue) {
        this.internalValue = newValue;
    };
    SearchBoxBase.prototype.onValueChanged = function (value) {
        this.$emit('input', value);
        this.$emit('change', value);
    };
    SearchBoxBase.prototype.onKeyDown = function (ev) {
        switch (ev.which) {
            case KeyCodes.escape:
                this.$emit('escape', ev);
                if (ev.defaultPrevented)
                    return;
                else
                    this.clearInput();
                break;
            case KeyCodes.enter:
                this.$emit('search', this.internalValue);
                return;
            default:
                if (!ev.defaultPrevented) {
                    return;
                }
        }
        // We only get here if the keypress has been handled,
        // or preventDefault was called in case of default keyDown handler
        ev.preventDefault();
        ev.stopPropagation();
    };
    SearchBoxBase.prototype.submit = function () {
        this.$emit('search', this.internalValue);
    };
    SearchBoxBase.prototype.onEscape = function (e) {
        this.$emit('escape', e);
        if (e.defaultPrevented)
            return;
        this.clearInput();
    };
    SearchBoxBase.prototype.clearInput = function (e) {
        this.$emit('clear', e);
        if (e && e.defaultPrevented)
            return;
        this.internalValue = '';
        this.$refs.input.focus();
    };
    SearchBoxBase.prototype.onFocus = function (e) {
        this.$emit('focus', e);
        if (e.defaultPrevented)
            return;
        this.isActive = true;
    };
    SearchBoxBase.prototype.onBlur = function (e) {
        this.$emit('blur', e);
        if (e.defaultPrevented)
            return;
        this.isActive = false;
    };
    SearchBoxBase.prototype.onInput = function (e) {
        this.internalValue = e.target.value;
    };
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SearchBoxBase.prototype, "underlined", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], SearchBoxBase.prototype, "defaultValue", void 0);
    __decorate([
        Prop({ type: String, default: 'Search' }),
        __metadata("design:type", String)
    ], SearchBoxBase.prototype, "placeholder", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], SearchBoxBase.prototype, "value", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SearchBoxBase.prototype, "disableAnimation", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SearchBoxBase.prototype, "disabled", void 0);
    __decorate([
        Prop({ type: String, default: 'Search' }),
        __metadata("design:type", String)
    ], SearchBoxBase.prototype, "iconName", void 0);
    __decorate([
        Watch('value'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [String]),
        __metadata("design:returntype", void 0)
    ], SearchBoxBase.prototype, "onPropValueChanged", null);
    __decorate([
        Watch('internalValue'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [String]),
        __metadata("design:returntype", void 0)
    ], SearchBoxBase.prototype, "onValueChanged", null);
    SearchBoxBase = __decorate([
        Component({
            components: { Icon: Icon, IconButton: IconButton },
            inheritAttrs: false,
        })
    ], SearchBoxBase);
    return SearchBoxBase;
}(BaseComponent));
export { SearchBoxBase };
//# sourceMappingURL=SearchBox.base.js.map