import { __decorate, __extends, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import { Label } from '../Label';
import BaseComponent from '../BaseComponent';
import { KeyCodes, classNamesFunction } from '@uifabric-vue/utilities';
var getClassNames = classNamesFunction();
export var ONKEYDOWN_TIMEOUT_DURATION = 1000;
var SliderBase = /** @class */ (function (_super) {
    __extends(SliderBase, _super);
    function SliderBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.internalValue = _this.value || _this.defaultValue || _this.min;
        _this.renderedValue = _this.value || _this.defaultValue || _this.min;
        _this.onKeyDownTimer = -1;
        return _this;
    }
    Object.defineProperty(SliderBase.prototype, "classNames", {
        get: function () {
            var _a = this, className = _a.className, disabled = _a.disabled, vertical = _a.vertical, renderedValue = _a.renderedValue, internalValue = _a.internalValue, showValue = _a.showValue, theme = _a.theme;
            return getClassNames(this.styles, {
                className: className,
                disabled: disabled,
                vertical: vertical,
                showTransitions: renderedValue === internalValue,
                showValue: showValue,
                theme: theme,
            });
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SliderBase.prototype, "thumbOffsetPercent", {
        get: function () {
            var _a = this, min = _a.min, max = _a.max, renderedValue = _a.renderedValue;
            return min === max ? 0 : ((renderedValue - min) / (max - min)) * 100;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SliderBase.prototype, "zeroOffsetPercent", {
        get: function () {
            var _a = this, min = _a.min, max = _a.max;
            return min >= 0 ? 0 : (-min / (max - min)) * 100;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SliderBase.prototype, "lengthString", {
        get: function () {
            return this.vertical ? 'height' : 'width';
        },
        enumerable: false,
        configurable: true
    });
    SliderBase.prototype.onMouseDown = function (event) {
        if (this.disabled)
            return;
        if (event.type === 'mousedown') {
            window.addEventListener('mousemove', this.onMove, true);
            window.addEventListener('mouseup', this.onMouseUp, true);
        }
        else if (event.type === 'touchstart') {
            window.addEventListener('touchmove', this.onMove, true);
            window.addEventListener('touchend', this.onMouseUp, true);
        }
        this.onMove(event);
    };
    SliderBase.prototype.onMouseUp = function (event) {
        if (this.disabled)
            return;
        this.renderedValue = this.internalValue;
        window.removeEventListener('mousemove', this.onMove, true);
        window.removeEventListener('mouseup', this.onMouseUp, true);
        window.removeEventListener('touchmove', this.onMove, true);
        window.removeEventListener('touchend', this.onMouseUp, true);
    };
    SliderBase.prototype.onMove = function (event) {
        var _this = this;
        window.requestAnimationFrame(function () {
            var _a = _this, max = _a.max, min = _a.min, step = _a.step, vertical = _a.vertical;
            var steps = (max - min) / step;
            var sliderPositionRect = _this.$refs.sliderLine.getBoundingClientRect();
            var sliderLength = !vertical
                ? sliderPositionRect.width
                : sliderPositionRect.height;
            var stepLength = sliderLength / steps;
            var currentSteps;
            var distance;
            if (!vertical) {
                var left = _this.getPosition(event, vertical);
                distance = left - sliderPositionRect.left;
                currentSteps = distance / stepLength;
            }
            else {
                var bottom = _this.getPosition(event, vertical);
                distance = sliderPositionRect.bottom - bottom;
                currentSteps = distance / stepLength;
            }
            var internalValue;
            var renderedValue;
            // The value shouldn't be bigger than max or be smaller than min.
            if (currentSteps > Math.floor(steps)) {
                renderedValue = internalValue = max;
            }
            else if (currentSteps < 0) {
                renderedValue = internalValue = min;
            }
            else {
                renderedValue = min + step * currentSteps;
                internalValue = min + step * Math.round(currentSteps);
            }
            _this.updateValue(internalValue, renderedValue);
        });
    };
    SliderBase.prototype.getPosition = function (event, vertical) {
        var currentPosition;
        switch (event.type) {
            case 'mousedown':
            case 'mousemove':
                currentPosition = !vertical
                    ? event.clientX
                    : event.clientY;
                break;
            case 'touchstart':
            case 'touchmove':
                currentPosition = !vertical
                    ? event.touches[0].clientX
                    : event.touches[0].clientY;
                break;
        }
        return currentPosition;
    };
    SliderBase.prototype.updateValue = function (value, renderedValue) {
        var _a = this, step = _a.step, snapToStep = _a.snapToStep;
        var numDec = 0;
        if (isFinite(step)) {
            while (Math.round(step * Math.pow(10, numDec)) / Math.pow(10, numDec) !== step) {
                numDec++;
            }
        }
        // Make sure value has correct number of decimal places based on number of decimals in step
        var roundedValue = parseFloat(value.toFixed(numDec));
        var valueChanged = roundedValue !== this.internalValue;
        if (snapToStep) {
            renderedValue = roundedValue;
        }
        this.internalValue = roundedValue;
        this.renderedValue = renderedValue;
        this.$emit('input', roundedValue);
    };
    SliderBase.prototype.onKeyDown = function (event) {
        var value = this.internalValue;
        var _a = this, max = _a.max, min = _a.min, step = _a.step;
        var diff = 0;
        switch (event.which) {
            case KeyCodes.left:
            case KeyCodes.down:
                diff = -step;
                this.clearOnKeyDownTimer();
                this.setOnKeyDownTimer(event);
                break;
            case KeyCodes.right:
            case KeyCodes.up:
                diff = step;
                this.clearOnKeyDownTimer();
                this.setOnKeyDownTimer(event);
                break;
            case KeyCodes.home:
                value = min;
                break;
            case KeyCodes.end:
                value = max;
                break;
            default:
                return;
        }
        var newValue = Math.min(max, Math.max(min, value + diff));
        this.updateValue(newValue, newValue);
        event.preventDefault();
        event.stopPropagation();
    };
    SliderBase.prototype.clearOnKeyDownTimer = function () {
        clearTimeout(this.onKeyDownTimer);
    };
    ;
    SliderBase.prototype.setOnKeyDownTimer = function (event) {
        this.onKeyDownTimer = setTimeout(function () {
        }, ONKEYDOWN_TIMEOUT_DURATION);
    };
    ;
    SliderBase.prototype.render = function (h) {
        var _a, _b, _c, _d, _e, _f, _g;
        var _h = this, classNames = _h.classNames, disabled = _h.disabled, originFromZero = _h.originFromZero, vertical = _h.vertical, zeroOffsetPercent = _h.zeroOffsetPercent, thumbOffsetPercent = _h.thumbOffsetPercent, lengthString = _h.lengthString, internalValue = _h.internalValue, css = _h.css, label = _h.label;
        var $label = h(Label, { class: classNames.titleLabel }, label);
        var $sliderLine = h('div', {
            ref: 'sliderLine',
            class: classNames.line,
        }, [
            originFromZero && h('span', {
                class: classNames.zeroTick,
                style: (_a = {}, _a[vertical ? 'bottom' : 'left'] = zeroOffsetPercent + "%", _a),
            }),
            h('span', {
                class: classNames.thumb,
                style: (_b = {}, _b[vertical ? 'bottom' : 'left'] = thumbOffsetPercent + "%", _b),
            }),
            originFromZero && h('span', {
                class: css(classNames.lineContainer, classNames.inactiveSection),
                style: (_c = {}, _c[lengthString] = Math.min(thumbOffsetPercent, zeroOffsetPercent) + "%", _c),
            }),
            originFromZero && h('span', {
                class: css(classNames.lineContainer, classNames.activeSection),
                style: (_d = {}, _d[lengthString] = Math.abs(zeroOffsetPercent - thumbOffsetPercent) + "%", _d),
            }),
            originFromZero && h('span', {
                class: css(classNames.lineContainer, classNames.inactiveSection),
                style: (_e = {}, _e[lengthString] = Math.min(100 - thumbOffsetPercent, 100 - zeroOffsetPercent) + "%", _e),
            }),
            !originFromZero && h('span', {
                class: css(classNames.lineContainer, classNames.activeSection),
                style: (_f = {}, _f[lengthString] = thumbOffsetPercent + "%", _f),
            }),
            !originFromZero && h('span', {
                class: css(classNames.lineContainer, classNames.inactiveSection),
                style: (_g = {}, _g[lengthString] = 100 - thumbOffsetPercent + "%", _g),
            }),
        ]);
        var $valueLabel = h(Label, {
            class: classNames.valueLabel,
        }, this.$scopedSlots.value ? [
            this.$scopedSlots.value({ value: internalValue }),
        ] : "" + internalValue);
        var $container = h('div', { class: classNames.container }, [
            h('div', {
                class: classNames.slideBox,
                attrs: {
                    tabindenx: disabled ? undefined : 0,
                },
                on: {
                    mousedown: this.onMouseDown,
                    keydown: this.onKeyDown,
                },
            }, [
                $sliderLine,
            ]),
            $valueLabel,
        ]);
        return h('div', { class: classNames.root }, [
            $label,
            $container,
        ]);
    };
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], SliderBase.prototype, "label", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SliderBase.prototype, "disabled", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SliderBase.prototype, "vertical", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SliderBase.prototype, "snapToStep", void 0);
    __decorate([
        Prop({ type: Number, default: 0 }),
        __metadata("design:type", Number)
    ], SliderBase.prototype, "min", void 0);
    __decorate([
        Prop({ type: Number, default: 10 }),
        __metadata("design:type", Number)
    ], SliderBase.prototype, "max", void 0);
    __decorate([
        Prop({ type: Number, default: 1 }),
        __metadata("design:type", Number)
    ], SliderBase.prototype, "step", void 0);
    __decorate([
        Prop({ type: Number, default: null }),
        __metadata("design:type", Number)
    ], SliderBase.prototype, "value", void 0);
    __decorate([
        Prop({ type: Number, default: null }),
        __metadata("design:type", Number)
    ], SliderBase.prototype, "defaultValue", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SliderBase.prototype, "showValue", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], SliderBase.prototype, "originFromZero", void 0);
    SliderBase = __decorate([
        Component({
            components: { Label: Label },
        })
    ], SliderBase);
    return SliderBase;
}(BaseComponent));
export { SliderBase };
//# sourceMappingURL=Slider.base.js.map