import { __assign, __awaiter, __decorate, __extends, __generator, __metadata } from "tslib";
import { Component, Prop, Watch } from 'vue-property-decorator';
import { Label } from '../Label';
import BaseComponent from '../BaseComponent';
import { classNamesFunction } from '@uifabric-vue/utilities';
var getClassNames = classNamesFunction();
var TextFieldBase = /** @class */ (function (_super) {
    __extends(TextFieldBase, _super);
    function TextFieldBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isFocused = false;
        _this.internalValue = _this.value;
        return _this;
    }
    Object.defineProperty(TextFieldBase.prototype, "classNames", {
        get: function () {
            var _a = this, theme = _a.theme, className = _a.className, disabled = _a.disabled, focused = _a.isFocused, required = _a.required, multiline = _a.multiline, label = _a.label, borderless = _a.borderless, underlined = _a.underlined, resizable = _a.resizable, autoAdjustHeight = _a.autoAdjustHeight;
            return getClassNames(this.styles, {
                theme: theme,
                className: className,
                disabled: disabled,
                focused: focused,
                required: required,
                multiline: multiline,
                hasLabel: !!label,
                borderless: borderless,
                underlined: underlined,
                hasIcon: false,
                resizable: resizable !== false,
                hasErrorMessage: !!this.errorMessage,
                inputClassName: '',
                autoAdjustHeight: autoAdjustHeight,
            });
        },
        enumerable: false,
        configurable: true
    });
    TextFieldBase.prototype.mounted = function () {
        this.adjustInputHeight();
    };
    TextFieldBase.prototype.updated = function () {
        this.adjustInputHeight();
    };
    TextFieldBase.prototype.onPropValueChanged = function (newValue) {
        this.internalValue = newValue;
    };
    TextFieldBase.prototype.onMultilineChanged = function (newValue, oldValue) {
        return __awaiter(this, void 0, void 0, function () {
            var textElement, start, end;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        textElement = this.$refs.textElement;
                        start = textElement.selectionStart || 0;
                        end = textElement.selectionEnd || 0;
                        if (!((newValue !== oldValue) && this.isFocused)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.$nextTick()];
                    case 1:
                        _a.sent();
                        this.$refs.textElement.focus();
                        this.$refs.textElement.setSelectionRange(start, end);
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    TextFieldBase.prototype.adjustInputHeight = function () {
        if (this.$refs.textElement && this.autoAdjustHeight && this.multiline) {
            var textField = this.$refs.textElement;
            textField.style.height = '';
            textField.style.height = textField.scrollHeight + 'px';
        }
    };
    TextFieldBase.prototype.onFocus = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.isFocused = true;
                this.$refs.textElement.setSelectionRange(this.internalValue.length, this.internalValue.length);
                return [2 /*return*/];
            });
        });
    };
    TextFieldBase.prototype.onInput = function (ev, value) {
        this.internalValue = value;
        this.$emit('input', value);
        this.$emit('change', ev, value);
    };
    TextFieldBase.prototype.render = function (h) {
        var _this = this;
        var _a = this, classNames = _a.classNames, required = _a.required, label = _a.label, errorMessage = _a.errorMessage, disabled = _a.disabled, multiline = _a.multiline, internalValue = _a.internalValue, readonly = _a.readonly, placeholder = _a.placeholder, resizable = _a.resizable;
        var Component = multiline ? 'textarea' : 'input';
        var labelStyles = classNames.subComponentStyles
            ? classNames.subComponentStyles.label
            : undefined;
        var id = "TextField" + this.uid;
        var $label = label && h(Label, {
            attrs: {
                for: id,
                required: required,
            },
            props: {
                styles: labelStyles,
            },
        }, label);
        var $fieldGroup = h('div', { class: classNames.fieldGroup }, [
            h(Component, {
                ref: 'textElement',
                class: classNames.field,
                attrs: __assign(__assign({ id: this.$attrs.id || id, value: internalValue }, this.$attrs), { disabled: disabled, readonly: readonly, required: required, placeholder: placeholder, rows: +this.$attrs.rows || 1, type: 'text', autocomplete: 'off' }),
                style: { resize: (resizable === false) && 'none' },
                on: {
                    focus: this.onFocus,
                    blur: function () { return (_this.isFocused = false); },
                    input: function (ev) { return _this.onInput(ev, ev.target.value); },
                },
            }, internalValue),
        ]);
        var $errorMessage = errorMessage && h('div', { class: classNames.description }, [
            h('div', { attrs: { role: 'alert' } }, [
                h('p', { class: classNames.errorMessage }, [
                    h('span', errorMessage),
                ]),
            ]),
        ]);
        return h('div', { class: classNames.root }, [
            h('div', { class: classNames.wrapper }, [
                $label,
                $fieldGroup,
            ]),
            $errorMessage,
        ]);
    };
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "multiline", void 0);
    __decorate([
        Prop({ type: Boolean, default: null }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "resizable", void 0);
    __decorate([
        Prop({ type: Boolean, default: null }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "autoAdjustHeight", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "borderless", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "disabled", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "underlined", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "readonly", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], TextFieldBase.prototype, "required", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], TextFieldBase.prototype, "label", void 0);
    __decorate([
        Prop({ type: String, default: '' }),
        __metadata("design:type", String)
    ], TextFieldBase.prototype, "value", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], TextFieldBase.prototype, "errorMessage", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], TextFieldBase.prototype, "placeholder", void 0);
    __decorate([
        Prop({ type: String, default: null }),
        __metadata("design:type", String)
    ], TextFieldBase.prototype, "description", void 0);
    __decorate([
        Watch('value'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [String]),
        __metadata("design:returntype", void 0)
    ], TextFieldBase.prototype, "onPropValueChanged", null);
    __decorate([
        Watch('multiline'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Boolean, Boolean]),
        __metadata("design:returntype", Promise)
    ], TextFieldBase.prototype, "onMultilineChanged", null);
    TextFieldBase = __decorate([
        Component({
            components: { Label: Label },
            inheritAttrs: false,
        })
    ], TextFieldBase);
    return TextFieldBase;
}(BaseComponent));
export { TextFieldBase };
//# sourceMappingURL=TextField.base.js.map